@@ -4,6 +4,8 @@ use std::{cell::RefCell, rc::Rc};
 use wasm_bindgen::closure::Closure;
 use web_sys::Node;
 
+use crate::dom::Measurements;
+
 /// Holds the user App and the dom updater
 /// This is passed into the event listener and the dispatch program
 /// will be called after the event is triggered.
@@ -49,17 +51,18 @@ where
         program
     }
 
-    /// Create a Program with an already `Rc,RefCell` wrapped of the app.
-    /// This is used for multiple reference for the same app to have it's own rendering function
-    pub fn from_rc(app: Rc<RefCell<APP>>, root_node: &Node) -> Self {
-        let dom_updater: DomUpdater<MSG> =
-            DomUpdater::new(app.borrow().view(), root_node);
-        let program = Program {
-            app,
-            dom_updater: Rc::new(RefCell::new(dom_updater)),
-        };
-        program.init_emit();
-        program
+    /// Map this program such that Program<APP,MSG> becomes Program<APP2,MSG>
+    pub fn map_program<F, APP2>(
+        &self,
+        func: F,
+        root_node: &Node,
+    ) -> Program<APP2, MSG>
+    where
+        F: Fn(&APP) -> APP2,
+        APP2: Component<MSG> + 'static,
+    {
+        let app2 = func(&*self.app.borrow());
+        Program::new(app2, root_node)
     }
 
     fn init_emit(&self) {
@@ -84,7 +87,7 @@ where
     }
 
     /// Creates an Rc wrapped instance of Program and replace the root_node with the app view
-    pub fn new_replace_mount(app: APP, root_node: &Node) -> Self {
+    pub fn replace_mount(app: APP, root_node: &Node) -> Self {
         let program = Self::new(app, root_node);
         program.start_replace_mount();
         program.after_mounted();
@@ -92,7 +95,7 @@ where
     }
 
     ///  Instantiage an app and append the view to the root_node
-    pub fn new_append_to_mount(app: APP, root_node: &Node) -> Self {
+    pub fn append_to_mount(app: APP, root_node: &Node) -> Self {
         let program = Self::new(app, root_node);
         program.start_append_to_mount();
         program.after_mounted();
@@ -101,9 +104,13 @@ where
 
     /// Instantiate the app and then append it to the document body
     pub fn mount_to_body(app: APP) -> Self {
-        Self::new_append_to_mount(app, &crate::body())
+        Self::append_to_mount(app, &crate::body())
     }
 
+    /// Replace the body of the document with the app
+    pub fn replace_body(app: APP) -> Self {
+        Self::replace_mount(app, &crate::body())
+    }
     fn start_append_to_mount(&self) {
         self.dom_updater.borrow_mut().append_to_mount(self)
     }
@@ -119,33 +126,33 @@ where
     /// - The view is reconstructed with the new state of the app.
     /// - The dom is updated with the newly reconstructed view.
     fn dispatch_inner(&self, msg: MSG) {
-        #[cfg(feature = "with-measure")]
         let t1 = crate::now();
         // update the app and emit the cmd returned from the update
         let cmd = self.app.borrow_mut().update(msg);
 
         if cmd.should_update_view {
             //trace!("Executing cmd..");
+            let t2 = crate::now();
+
             #[cfg(feature = "with-measure")]
-            let t2 = {
-                let t2 = crate::now();
-                log::trace!("app update took: {}ms", t2 - t1);
-                t2
-            };
+            log::trace!("app update took: {}ms", t2 - t1);
+
             // a new view is created due to the app update
             let view = self.app.borrow().view();
+            let node_count = view.node_count();
+            let t3 = crate::now();
+
             #[cfg(feature = "with-measure")]
-            let t3 = {
-                let t3 = crate::now();
-                log::trace!("creating app view took: {}ms", t3 - t2);
-                t3
-            };
+            log::trace!("creating app view took: {}ms", t3 - t2);
+
             // update the last DOM node tree with this new view
             self.dom_updater.borrow_mut().update_dom(self, view);
+            let t4 = crate::now();
+            #[cfg(feature = "with-measure")]
+            log::trace!("dom update took: {}ms", t4 - t3);
+
             #[cfg(feature = "with-measure")]
             {
-                let t4 = crate::now();
-                log::trace!("dom update took: {}ms", t4 - t3);
                 let dispatch_duration = t4 - t1;
                 // 60fps is 16.667 ms per frame.
                 if dispatch_duration > 16.0 {
@@ -153,9 +160,27 @@ where
                 } else {
                     log::trace!("dispatch took: {}ms", dispatch_duration);
                 }
-            };
+            }
+
+            if cmd.log_measurements {
+                let measurements = Measurements {
+                    view_node_count: node_count,
+                    update_dispatch_took: t2 - t1,
+                    build_view_took: t3 - t2,
+                    dom_update_took: t4 - t3,
+                    total_time: t4 - t1,
+                };
+                // tell the app on app performance measurements
+                let mut cmd_measurement =
+                    self.app.borrow().measurements(measurements);
+                cmd_measurement.should_update_view = false;
+                cmd_measurement.emit(self);
+            } else {
+                #[cfg(any(feature = "with-debug", feature = "with-measure"))]
+                log::info!("skipped logging measurements");
+            }
         } else {
-            #[cfg(feature = "with-debug")]
+            #[cfg(any(feature = "with-debug", feature = "with-measure"))]
             log::info!("dom update is skipped here");
         }
         cmd.emit(self);
